using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using SubSonic;

namespace ElegantCode
{
	public class ExportToExcelOptions
	{
		private string _fieldDelimiter = ",";
		private string _lineDelimiter = "\n";
		private string _textQualifier = "\"";
		private string _textQualifierReplacement = "'";
		private string _fieldDelimiterReplacement = ",";

		public bool _includeHeaderRow = true;
		public List<string> _excludeColumns = new List<string>();

		public string FieldDelimiter { get { return _fieldDelimiter; } set { _fieldDelimiter = value; } }
		public string LineDelimiter { get { return _lineDelimiter; } set { _lineDelimiter = value; } }
		public bool IncludeHeaderRow { get { return _includeHeaderRow; } set { _includeHeaderRow = value; } }
		public List<string> ExcludeColumns { get { return _excludeColumns; } }
		public string TextQualifier { get { return _textQualifier; } set { _textQualifier = value; } }
		public string FieldDelimiterReplacement { get { return _fieldDelimiterReplacement; } set { _fieldDelimiterReplacement = value; } }
		public string TextQualifierReplacement { get { return _textQualifierReplacement; } set { _textQualifierReplacement = value; } }
	}

	public static class ExportToExcelExtension
	{
		public static string CreateCsvData<ItemType, ListType>(this ActiveList<ItemType, ListType> list)
			where ItemType
		: ActiveRecord<ItemType>, new()
			where ListType
		: AbstractList<ItemType, ListType>, new()
		{
			return CreateCsvData(list, new ExportToExcelOptions());
		}


		public static string CreateCsvData<ItemType, ListType>(this ActiveList<ItemType, ListType> list, ExportToExcelOptions options)
			where ItemType
				: ActiveRecord<ItemType>, new()
			where ListType
				: AbstractList<ItemType, ListType>, new()
		{
			DataTable dt = list.ToDataTable();
			StringBuilder sb = new StringBuilder();
			GetCsvHeader( dt, sb, options );
			GetCsvData( dt, sb, options );
			return sb.ToString();
		}

		private static List<string> GetValidColumns(DataTable dt, ExportToExcelOptions options)
		{
			List<string> list = new List<string>();
			foreach (DataColumn column in dt.Columns)
			{
				if (!options.ExcludeColumns.Exists( delegate(string s) { return s == column.ColumnName; } ))
					list.Add( column.ColumnName );
			}
			return list;
		}


		private static void GetCsvData(DataTable dt, StringBuilder sb, ExportToExcelOptions options)
		{
			List<string> columns = GetValidColumns( dt, options );

			foreach (DataRow row in dt.Rows)
			{
				for (int i = 0; i < columns.Count; i++)
				{
					string column = columns[i];
					string data = FormatText( row[column].ToString(), options );
					string finalizer = (i < columns.Count - 1) ? options.FieldDelimiter : options.LineDelimiter;
					sb.Append( data + finalizer );
				}
			}
		}

		private static string FormatText(string text, ExportToExcelOptions options)
		{
			if (options.FieldDelimiter != options.FieldDelimiterReplacement)
				text = text.Replace( options.FieldDelimiter, options.FieldDelimiterReplacement );
			if (options.TextQualifier != options.TextQualifierReplacement)
				text = text.Replace( options.TextQualifier, options.TextQualifierReplacement );
			return options.TextQualifier + text + options.TextQualifier;
		}

		private static void GetCsvHeader(DataTable dt, StringBuilder sb, ExportToExcelOptions options)
		{
			if (!options.IncludeHeaderRow)
				return;
			List<string> columnList = GetValidColumns( dt, options );
			for (int i = 0; i < columnList.Count; i++)
			{
				string column = options.TextQualifier + columnList[i] + options.TextQualifier;
				string finalizer = (i < columnList.Count - 1) ? options.FieldDelimiter : options.LineDelimiter;
				sb.Append( column + finalizer );
			}
		}

	}
}